@@ -7,13 +7,6 @@ use std::{
 };
 
 use crate::{generate_builder_method, LibSDBootConfError};
-
-#[derive(Default, Debug)]
-pub struct Entry {
-    pub id: String,
-    pub tokens: Vec<Token>,
-}
-
 #[derive(Debug)]
 pub enum Token {
     /// Text to show in the menu.
@@ -32,6 +25,47 @@ pub enum Token {
     Initrd(PathBuf),
 }
 
+impl FromStr for Token {
+    type Err = LibSDBootConfError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let mut parts = s.splitn(2, ' ');
+        let key = parts.next().ok_or(LibSDBootConfError::EntryParseError)?;
+        let value = parts.next().ok_or(LibSDBootConfError::EntryParseError)?;
+
+        Ok(match key {
+            "title" => Self::Title(value.to_owned()),
+            "version" => Self::Version(value.to_owned()),
+            "machine-id" => Self::MachineID(value.to_owned()),
+            "efi" => Self::Efi(PathBuf::from(value)),
+            "options" => Self::Options(value.to_owned()),
+            "linux" => Self::Linux(PathBuf::from(value)),
+            "initrd" => Self::Initrd(PathBuf::from(value)),
+            _ => return Err(LibSDBootConfError::InvalidToken(key.to_owned())),
+        })
+    }
+}
+
+impl ToString for Token {
+    fn to_string(&self) -> String {
+        match self {
+            Self::Title(title) => format!("title {}\n", title),
+            Self::Version(version) => format!("version {}\n", version),
+            Self::MachineID(machine_id) => format!("machine-id {}\n", machine_id),
+            Self::Efi(efi) => format!("efi {}\n", efi.display()),
+            Self::Options(options) => format!("options {}\n", options),
+            Self::Linux(linux) => format!("linux {}\n", linux.display()),
+            Self::Initrd(initrd) => format!("initrd {}\n", initrd.display()),
+        }
+    }
+}
+
+#[derive(Default, Debug)]
+pub struct Entry {
+    pub id: String,
+    pub tokens: Vec<Token>,
+}
+
 impl FromStr for Entry {
     type Err = LibSDBootConfError;
 
@@ -44,20 +78,7 @@ impl FromStr for Entry {
                 continue;
             }
 
-            let mut parts = line.splitn(2, ' ');
-            let key = parts.next().ok_or(LibSDBootConfError::EntryParseError)?;
-            let value = parts.next().ok_or(LibSDBootConfError::EntryParseError)?;
-
-            entry.tokens.push(match key {
-                "title" => Token::Title(value.to_owned()),
-                "version" => Token::Version(value.to_owned()),
-                "machine-id" => Token::MachineID(value.to_owned()),
-                "efi" => Token::Efi(PathBuf::from(value)),
-                "options" => Token::Options(value.to_owned()),
-                "linux" => Token::Linux(PathBuf::from(value)),
-                "initrd" => Token::Initrd(PathBuf::from(value)),
-                _ => continue,
-            })
+            entry.tokens.push(line.parse()?);
         }
 
         Ok(entry)
@@ -69,15 +90,7 @@ impl ToString for Entry {
         let mut s = String::new();
 
         for token in &self.tokens {
-            s.push_str(&match token {
-                Token::Title(title) => format!("title {}\n", title),
-                Token::Version(version) => format!("version {}\n", version),
-                Token::MachineID(machine_id) => format!("machine-id {}\n", machine_id),
-                Token::Efi(efi) => format!("efi {}\n", efi.display()),
-                Token::Options(options) => format!("options {}\n", options),
-                Token::Linux(linux) => format!("linux {}\n", linux.display()),
-                Token::Initrd(initrd) => format!("initrd {}\n", initrd.display()),
-            })
+            s.push_str(&token.to_string())
         }
 
         s
@@ -101,10 +114,14 @@ impl Entry {
     /// assert_eq!(entry.id, "5.12.0-aosc-main");
     /// println!("{:?}", entry.tokens); // [Token::Title("title 5.12.0-aosc-main")]
     /// ```
-    pub fn new(id: impl Into<String>, tokens: impl Into<Vec<Token>>) -> Entry {
+    pub fn new<S, T>(id: S, tokens: T) -> Entry
+    where
+        S: Into<String>,
+        T: IntoIterator<Item = Token>,
+    {
         Entry {
             id: id.into(),
-            tokens: tokens.into(),
+            tokens: tokens.into_iter().collect(),
         }
     }
 
@@ -117,7 +134,7 @@ impl Entry {
     ///
     /// let entry = Entry::load("/path/to/config").unwrap();
     /// ```
-    pub fn load(path: impl AsRef<Path>) -> Result<Entry, LibSDBootConfError> {
+    pub fn load<P: AsRef<Path>>(path: P) -> Result<Entry, LibSDBootConfError> {
         let id = path
             .as_ref()
             .file_name()
@@ -147,7 +164,7 @@ impl Entry {
     /// );
     /// entry.write("/path/to/entry").unwrap();
     /// ```
-    pub fn write(&self, path: impl AsRef<Path>) -> Result<(), LibSDBootConfError> {
+    pub fn write<P: AsRef<Path>>(&self, path: P) -> Result<(), LibSDBootConfError> {
         fs::write(path, self.to_string())?;
 
         Ok(())
@@ -162,19 +179,40 @@ pub struct EntryBuilder {
 
 impl EntryBuilder {
     /// Build an empty EntryBuilder with an entry id.
-    pub fn new(id: impl Into<String>) -> Self {
+    pub fn new<S: Into<String>>(id: S) -> Self {
         Self {
             entry: Entry::new(id, Vec::new()),
         }
     }
 
-    generate_builder_method!(token title, impl Into<String>, Title);
-    generate_builder_method!(token version, impl Into<String>, Version);
-    generate_builder_method!(token machine_id, impl Into<String>, MachineID);
-    generate_builder_method!(token efi, impl Into<PathBuf>, Efi);
-    generate_builder_method!(token options, impl Into<String>, Options);
-    generate_builder_method!(token linux, impl Into<PathBuf>, Linux);
-    generate_builder_method!(token initrd, impl Into<PathBuf>, Initrd);
+    generate_builder_method!(
+        /// Add a `Title` to the entry.
+        token => Title REAL(entry) title(S => String)
+    );
+    generate_builder_method!(
+        /// Add a `Version` to the entry.
+        token => Version REAL(entry) version(S => String)
+    );
+    generate_builder_method!(
+        /// Add a `MachineID` to the entry.
+        token => MachineID REAL(entry) machine_id(S => String)
+    );
+    generate_builder_method!(
+        /// Add an `Efi` to the entry.
+        token => Efi REAL(entry) efi(P => PathBuf)
+    );
+    generate_builder_method!(
+        /// Add an `Options` to the entry.
+        token => Options REAL(entry) options(S => String)
+    );
+    generate_builder_method!(
+        /// Add a `Linux` to the entry.
+        token => Linux REAL(entry) linux(P => PathBuf)
+    );
+    generate_builder_method!(
+        /// Add an `Initrd` to the entry.
+        token => Initrd REAL(entry) initrd(P => PathBuf)
+    );
 
     /// Build the Entry.
     pub fn build(self) -> Entry {
